@@ -1,5 +1,5 @@
 /*
- * Copyright 2009-2019 Peter Kosyh <p.kosyh at gmail.com>
+ * Copyright 2009-2022 Peter Kosyh <p.kosyh at gmail.com>
  *
  * Permission is hereby granted, free of charge, to any person
  * obtaining a copy of this software and associated documentation files
@@ -322,6 +322,16 @@ static char *game_version(const char *path, const char *d_name)
 	trunc_lines(p, 0);
 	return p;
 }
+
+static char *game_direction(const char *path, const char *d_name)
+{
+	char *p = game_tag(path, d_name, "Direction");
+	if (!p)
+		return strdup(d_name);
+	trunc_lines(p, 0);
+	return p;
+}
+
 #if 0
 static char *game_api(const char *path, const char *d_name)
 {
@@ -338,6 +348,7 @@ static void game_info_free(struct game *g)
 	FREE(g->info);
 	FREE(g->author);
 	FREE(g->version);
+	FREE(g->direction);
 /*	FREE(g->api); */
 }
 
@@ -354,6 +365,7 @@ static void game_info_fill(struct game *g)
 	g->info = game_info(dirpath(g->path), g->dir);
 	g->author = game_author(dirpath(g->path), g->dir);
 	g->version = game_version(dirpath(g->path), g->dir);
+	g->direction = game_direction(dirpath(g->path), g->dir);
 /*	g->api = game_api(dirpath(g->path), g->dir); */
 }
 
@@ -545,7 +557,11 @@ void game_clear(int x, int y, int w, int h)
 
 void game_clear_all(void)
 {
-	game_clear(0, 0, game_theme.w, game_theme.h);
+	if (DIRECT_MODE)
+		gfx_img_fill(gfx_screen(NULL), 0, 0, game_theme.w, game_theme.h,
+			gfx_col(game_theme.brdcol.r, game_theme.brdcol.g, game_theme.brdcol.b));
+	else
+		game_clear(0, 0, game_theme.w, game_theme.h);
 }
 
 void game_clear(int x, int y, int w, int h);
@@ -747,8 +763,17 @@ int game_apply_theme(void)
 	int align = game_theme.win_align;
 	layout_t lay = NULL;
 	textbox_t box = NULL;
-
-	gfx_bg(game_theme.bgcol);
+	struct game *g = game_lookup(curgame_dir);
+	int rtl = 0;
+	if (g && g->direction) {
+		if (!strcmp(g->direction, "rtl"))
+			rtl = 1;
+		else if (!strcmp(g->direction, "auto"))
+			rtl = -1;
+	}
+	gfx_bg(game_theme.xoff, game_theme.yoff,
+		game_theme.w - 2*game_theme.xoff, game_theme.h - 2*game_theme.yoff,
+		game_theme.bgcol, game_theme.brdcol);
 	if (!DIRECT_MODE)
 		game_clear_all();
 	gfx_flip();
@@ -769,6 +794,8 @@ int game_apply_theme(void)
 		/* txt_layout_link_style(lay, 4); */
 		txt_layout_active_color(lay, game_theme.acol);
 		txt_layout_font_height(lay, game_theme.font_height);
+		txt_layout_rtl(lay, rtl);
+
 		txt_box_set(box, lay);
 		el_set(el_scene, elt_box, game_theme.win_x, 0, box);
 	}
@@ -783,6 +810,7 @@ int game_apply_theme(void)
 			txt_layout_link_color(lay, game_theme.ilcol);
 			txt_layout_active_color(lay, game_theme.iacol);
 			txt_layout_font_height(lay, game_theme.inv_font_height);
+			txt_layout_rtl(lay, rtl);
 
 			box = txt_box(game_theme.inv_w, game_theme.inv_h);
 			if (!box)
@@ -802,6 +830,7 @@ int game_apply_theme(void)
 		txt_layout_link_color(lay, game_theme.lcol);
 		txt_layout_active_color(lay, game_theme.acol);
 		txt_layout_font_height(lay, game_theme.font_height);
+		txt_layout_rtl(lay, rtl);
 
 		el_set(el_title, elt_layout, game_theme.win_x, game_theme.win_y, lay);
 	}
@@ -815,6 +844,7 @@ int game_apply_theme(void)
 		txt_layout_link_color(lay, game_theme.lcol);
 		txt_layout_active_color(lay, game_theme.acol);
 		txt_layout_font_height(lay, game_theme.font_height);
+		txt_layout_rtl(lay, rtl);
 
 		el_set(el_ways, elt_layout, game_theme.win_x, 0, lay);
 	}
@@ -878,7 +908,7 @@ void game_gfx_commit(int sync)
 
 static int game_render_callback_redraw(void);
 
-static void anigif_do(void *data)
+static void anim_do(void *data)
 {
 	void *v;
 	img_t img;
@@ -888,27 +918,27 @@ static void anigif_do(void *data)
 
 	game_cursor(CURSOR_CLEAR);
 
-	if (gfx_frame_gif(el_img(el_spic))) { /* scene */
+	if (gfx_frame_anim(el_img(el_spic))) { /* scene */
 		game_render_callback_redraw();
-		gfx_update_gif(el_img(el_spic), game_update);
+		gfx_update_anim(el_img(el_spic), game_update);
 	}
 
 	game_cursor(CURSOR_CLEAR);
 
 	for (v = NULL; (img = txt_layout_images(txt_box_layout(el_box(el_scene)), &v)); ) { /* scene */
 		game_cursor(CURSOR_CLEAR);
-		if ((img != el_img(el_spic)) && gfx_frame_gif(img)) {
+		if ((img != el_img(el_spic)) && gfx_frame_anim(img)) {
 			game_render_callback_redraw();
-			gfx_update_gif(img, game_update);
+			gfx_update_anim(img, game_update);
 		}
 	}
 	game_cursor(CURSOR_CLEAR);
 
 	for (v = NULL; (img = txt_layout_images(txt_box_layout(el_box(el_inv)), &v)); ) { /* inv */
 		game_cursor(CURSOR_CLEAR);
-		if (gfx_frame_gif(img)) {
+		if (gfx_frame_anim(img)) {
 			game_render_callback_redraw();
-			gfx_update_gif(img, game_update);
+			gfx_update_anim(img, game_update);
 		}
 	}
 
@@ -916,9 +946,9 @@ static void anigif_do(void *data)
 
 	for (v = NULL; (img = txt_layout_images(el_layout(el_title), &v)); ) { /* title */
 		game_cursor(CURSOR_CLEAR);
-		if (gfx_frame_gif(img)) {
+		if (gfx_frame_anim(img)) {
 			game_render_callback_redraw();
-			gfx_update_gif(img, game_update);
+			gfx_update_anim(img, game_update);
 		}
 	}
 
@@ -926,9 +956,9 @@ static void anigif_do(void *data)
 
 	for (v = NULL; (img = txt_layout_images(el_layout(el_ways), &v)); ) { /* ways */
 		game_cursor(CURSOR_CLEAR);
-		if (gfx_frame_gif(img)) {
+		if (gfx_frame_anim(img)) {
 			game_render_callback_redraw();
-			gfx_update_gif(img, game_update);
+			gfx_update_anim(img, game_update);
 		}
 	}
 	game_cursor(CURSOR_ON);
@@ -944,8 +974,8 @@ int counter_fn(int interval, void *p)
 		push_user_event(NULL, NULL);
 	}
 #endif
-	if (gfx_is_drawn_gifs() && !DIRECT_MODE)
-		push_user_event(anigif_do, NULL);
+	if (gfx_is_drawn_anims() && !DIRECT_MODE)
+		push_user_event(anim_do, NULL);
 	return interval;
 }
 
@@ -1004,7 +1034,6 @@ int game_use_theme(void)
 	game_theme_changed = 0;
 	game_own_theme = 0;
 
-	game_theme.changed = CHANGED_ALL;
 	memset(objs, 0, sizeof(struct el) * el_max);
 
 	if (game_default_theme()) {
@@ -1200,12 +1229,12 @@ static int _el_clear(int n, clear_fn clear)
 		clear(x, y, w, h);
 	if (o->type == elt_box) {
 		for (v = NULL; (img = txt_layout_images(txt_box_layout(el_box(n)), &v)); )
-			gfx_dispose_gif(img);
+			gfx_dispose_anim(img);
 	} else if (o->type == elt_layout) {
 		for (v = NULL; (img = txt_layout_images(el_layout(n), &v)); )
-			gfx_dispose_gif(img);
+			gfx_dispose_anim(img);
 	} else if (o->type == elt_image)
-		gfx_dispose_gif(el_img(n));
+		gfx_dispose_anim(el_img(n));
 
 	return 1;
 }
@@ -1337,6 +1366,7 @@ void box_update_scrollbar(int n)
 	el_update(eldown->id);
 }
 
+/* Decides what to draw */
 void el_draw(int n)
 {
 	int x, y;
@@ -1376,19 +1406,21 @@ img_t	game_pict_scale(img_t img, int ww, int hh)
 		return img; /* do not scale sprites! */
 	}
 
-	if (game_theme.scale > 1.0f)
-		theme_img_scale(&img);
+	if (game_theme.img_scale * game_theme.scale > 1.0f)
+		scale = game_theme.img_scale * game_theme.scale;
 
-	w = gfx_img_w(img);
-	h = gfx_img_h(img);
+	w = gfx_img_w(img) * scale;
+	h = gfx_img_h(img) * scale;
 
 	if (ww == -1)
 		ww = w;
 	if (hh == -1)
 		hh = h;
 
-	if (w <= ww && h <= hh)
+	if (w <= ww && h <= hh) {
+		theme_img_scale(&img);
 		return img;
+	}
 
 	www = ww;
 	hhh = hh;
@@ -1404,6 +1436,8 @@ img_t	game_pict_scale(img_t img, int ww, int hh)
 	}
 	if (scale < 0)
 		scale = 0;
+	if (game_theme.img_scale * game_theme.scale > 1.0f)
+		scale *= (game_theme.img_scale * game_theme.scale);
 	img2 = gfx_scale(img, scale, scale, SCALABLE_THEME_SMOOTH);
 	gfx_free_image(img);
 	return img2;
@@ -1841,7 +1875,7 @@ static void after_cmd(void)
 
 static void after_fading(void *aux)
 {
-	gfx_start_gif(el_img(el_spic));
+	gfx_start_anim(el_img(el_spic));
 	gfx_free_image(fade_ctx.offscreen);
 	game_render_callback_redraw();
 	after_click(fade_ctx.flags, fade_ctx.m_restore);
@@ -2213,7 +2247,7 @@ inv:
 	if (fading) {
 		img_t offscreen;
 		game_cursor(CURSOR_OFF);
-		gfx_stop_gif(el_img(el_spic));
+		gfx_stop_anim(el_img(el_spic));
 		instead_render_callback();
 		offscreen = gfx_screen(oldscreen);
 		fade_ctx.offscreen = offscreen;
@@ -3159,6 +3193,16 @@ static int game_input(int down, const char *key, int x, int y, int mb)
 		k = (key) ? key:"";
 		args[1].val = (char*)k; args[1].type = INSTEAD_STR;
 		args[2].val = NULL;
+	} else if (mb == EV_CODE_WHEEL) {
+		if (!game_grab_events)
+			return -1;
+
+		args[0].val = "wheel"; args[0].type = INSTEAD_STR;
+		snprintf(tx, sizeof(tx), "%d", x);
+		snprintf(ty, sizeof(ty), "%d", y);
+		args[1].val = tx; args[1].type = INSTEAD_NUM;
+		args[2].val = ty; args[2].type = INSTEAD_NUM;
+		args[3].val = NULL;
 	} else {
 		const char *k;
 		int px = -1;
@@ -3221,8 +3265,6 @@ int game_from_disk(void)
 	char *g, *p, *b, *d;
 	char dir[PATH_MAX];
 	char base[PATH_MAX];
-#ifndef MAEMO
-#ifndef S60
 	if (opt_fs) {
 		int old_menu = (menu_shown) ? cur_menu: -1;
 		opt_fs ^= 1;
@@ -3230,8 +3272,6 @@ int game_from_disk(void)
 		if (old_menu != -1)
 			game_menu(old_menu);
 	}
-#endif
-#endif
 	mouse_cursor(1);
 	game_cursor(CURSOR_OFF);
 	browse_dialog = 1;
@@ -3316,6 +3356,12 @@ static int game_input_events(struct inp_event *ev)
 	} else if (ev->type == MOUSE_DOWN || ev->type == MOUSE_UP) {
 		if (!game_input((ev->type == MOUSE_DOWN), "mouse", ev->x, ev->y, ev->code))
 			return 1;
+	} else if (ev->type == MOUSE_WHEEL_DOWN) {
+		if (!game_input(1, "wheel", 0, ev->count, EV_CODE_WHEEL))
+			return 1;
+	} else if (ev->type == MOUSE_WHEEL_UP) {
+		if (!game_input(1, "wheel", 0, -ev->count, EV_CODE_WHEEL))
+			return 1;
 	} else if (ev->type == KEY_TEXT) {
 		if (!game_input(1, ev->sym, -1, -1, EV_CODE_TEXT))
 			return 1;
@@ -3347,17 +3393,6 @@ static int kbd_modifiers(struct inp_event *ev)
 static int is_key_back(struct inp_event *ev)
 {
 	if (!is_key(ev, "escape")
-#if defined(S60) || defined(_WIN32_WCE) || defined(WINRT)
-	    || !is_key(ev, "space")
-#endif
-#if defined(_WIN32_WCE) || defined(WINRT)
-	    || (ev->code >= 0xc0 && ev->code <= 0xcf) ||
-	    !is_key(ev, "f1") ||
-	    !is_key(ev, "f2") ||
-	    !is_key(ev, "f3") ||
-	    !is_key(ev, "f4") ||
-	    !is_key(ev, "f5")
-#endif
 #ifdef ANDROID
 	    || ev->code == 118
 #endif
@@ -3437,9 +3472,6 @@ static int kbd_instead(struct inp_event *ev, int *x, int *y)
 	} else if (DIRECT_MODE && !menu_shown) {
 		; /* nothing todo */
 	} else if (!alt_pressed && (!is_key(ev, "return") || !is_key(ev, "enter")
-#ifdef S60
-				    || !is_key(ev, ".")
-#endif
 				    )) {
 		gfx_cursor(x, y);
 		game_highlight(-1, -1, 0); /* reset */
@@ -3499,7 +3531,6 @@ static int kbd_instead(struct inp_event *ev, int *x, int *y)
 			else
 				game_scroll_pdown();
 		}
-#if !defined(S60) && !defined(_WIN32_WCE) && !defined(WINRT)
 	} else if (!is_key(ev, "left") || !is_key(ev, "[4]")) {
 		select_ref(1, 0);
 	} else if (!is_key(ev, "right") || !is_key(ev, "[6]")) {
@@ -3508,18 +3539,6 @@ static int kbd_instead(struct inp_event *ev, int *x, int *y)
 		scroll_pup(el_scene);
 	} else if (!is_key(ev, "space") && !menu_shown) {
 		scroll_pdown(el_scene);
-#else
-	} else if (!is_key(ev, "left") || !is_key(ev, "[4]")) {
-		if (menu_shown)
-			select_ref(1, 0);
-		else
-			select_frame(1);
-	} else if (!is_key(ev, "right") || !is_key(ev, "[6]")) {
-		if (menu_shown)
-			select_ref(0, 0);
-		else
-			select_frame(0);
-#endif
 	} else
 		return 0;
 	return 1;
