@@ -1,5 +1,5 @@
 /*
- * Copyright 2009-2020 Peter Kosyh <p.kosyh at gmail.com>
+ * Copyright 2009-2022 Peter Kosyh <p.kosyh at gmail.com>
  *
  * Permission is hereby granted, free of charge, to any person
  * obtaining a copy of this software and associated documentation files
@@ -29,16 +29,16 @@
 
 #define DATA_IDF INSTEAD_IDF
 #ifdef _USE_SDL
-#ifndef __EMSCRIPTEN__
+#if !defined(__EMSCRIPTEN__) && !defined(WINRT)
 static SDL_mutex *sem;
 #endif
 void instead_lock(void) {
-#ifndef __EMSCRIPTEN__
+#if !defined(__EMSCRIPTEN__) && !defined(WINRT)
 	SDL_LockMutex(sem);
 #endif
 }
 void instead_unlock(void) {
-#ifndef __EMSCRIPTEN__
+#if !defined(__EMSCRIPTEN__) && !defined(WINRT)
 	SDL_UnlockMutex(sem);
 #endif
 }
@@ -63,7 +63,8 @@ char 		*togame(const char *s);
 lua_State	*L = NULL;
 
 static char *err_msg = NULL;
-static char instead_api_path[PATH_MAX];
+static char instead_api_path[PATH_MAX + 1];
+static char instead_base_path[PATH_MAX] = STEAD_PATH;
 
 static char *API = NULL;
 static char *MAIN = NULL;
@@ -408,7 +409,7 @@ int instead_function(char *s, struct instead_args *args)
 				lua_pushnil(L);
 				break;
 			case INSTEAD_NUM:
-				lua_pushnumber(L, atoi(args->val));
+				lua_pushinteger(L, atoi(args->val));
 				break;
 			case INSTEAD_BOOL:
 				if (!strcmp(args->val, "true"))
@@ -442,11 +443,21 @@ int instead_function(char *s, struct instead_args *args)
 }
 
 #ifdef _HAVE_ICONV
-static char *curcp = "UTF-8";
+static char *curcp = NULL;
 static char *fromcp = NULL;
 #endif
 
 #ifdef _HAVE_ICONV
+void instead_set_encoding(const char *cp)
+{
+	if (curcp)
+		free(curcp);
+	if (cp)
+		curcp = strdup(cp);
+	else
+		curcp = NULL;
+	return;
+}
 char *instead_fromgame(const char *s)
 {
 	iconv_t han;
@@ -501,6 +512,9 @@ char *togame(const char *s)
 		return NULL;
 	return strdup(s);
 }
+void instead_set_encoding(const char *cp)
+{
+}
 #endif
 
 static int instead_getargs (char **argv, int n)
@@ -561,12 +575,12 @@ err:
 	return -1;
 }
 
-int instead_loadfile(char *name)
+int instead_loadfile(const char *name)
 {
 	return instead_loadscript(name, -1, NULL, 1);
 }
 
-int instead_loadscript(char *name, int argc, char **argv, int exec)
+int instead_loadscript(const char *name, int argc, char **argv, int exec)
 {
 	int status;
 	if (exec && argc >= 0)
@@ -689,6 +703,8 @@ static int luaB_dofile (lua_State *L) {
 	return lua_gettop(L) - n;
 }
 
+#if LUA_VERSION_NUM <= 503
+/* is this hack still needed? */
 static int luaB_print (lua_State *L) {
 	int n = lua_gettop(L);  /* number of arguments */
 	int i;
@@ -708,6 +724,22 @@ static int luaB_print (lua_State *L) {
 	fputs("\n", stdout);
 	return 0;
 }
+#else
+static int luaB_print (lua_State *L) {
+  int n = lua_gettop(L);  /* number of arguments */
+  int i;
+  for (i = 1; i <= n; i++) {  /* for each argument */
+    size_t l;
+    const char *s = luaL_tolstring(L, i, &l);  /* convert it to string */
+    if (i > 1)  /* not the first element? */
+      lua_writestring("\t", 1);  /* add a tab before it */
+    lua_writestring(s, l);  /* print it */
+    lua_pop(L, 1);  /* pop result */
+  }
+  lua_writeline();
+  return 0;
+}
+#endif
 
 static int luaB_maxn (lua_State *L) {
 	lua_Integer max = 0;
@@ -838,7 +870,7 @@ static int luaB_utf_next(lua_State *L) {
 		if (idx < len)
 			l = utf_ff(s + idx, s + len - 1);
 	}
-	lua_pushnumber(L, l);
+	lua_pushinteger(L, l);
 	return 1;
 }
 
@@ -856,7 +888,7 @@ static int luaB_utf_prev(lua_State *L) {
 				l = utf_bb(s, s + idx);
 		}
 	}
-	lua_pushnumber(L, l);
+	lua_pushinteger(L, l);
 	return 1;
 }
 
@@ -902,7 +934,7 @@ static int luaB_utf_len(lua_State *L) {
 			sym ++;
 		}
 	}
-	lua_pushnumber(L, sym);
+	lua_pushinteger(L, sym);
 	return 1;
 }
 
@@ -944,18 +976,12 @@ static int instead_platform(void)
 	snprintf(plat, sizeof(plat) - 1, "PLATFORM='IOS'");
 #elif defined(__APPLE__)
 	snprintf(plat, sizeof(plat) - 1, "PLATFORM='MACOSX'");
-#elif defined(_WIN32_WCE)
-	snprintf(plat, sizeof(plat) - 1, "PLATFORM='WINCE'");
 #elif defined(WINRT)
 	snprintf(plat, sizeof(plat) - 1, "PLATFORM='WINRT'");
-#elif defined(S60)
-	snprintf(plat, sizeof(plat) - 1, "PLATFORM='S60'");
 #elif defined(ANDROID)
 	snprintf(plat, sizeof(plat) - 1, "PLATFORM='ANDROID'");
 #elif defined(_WIN32)
 	snprintf(plat, sizeof(plat) - 1, "PLATFORM='WIN32'");
-#elif defined(MAEMO)
-	snprintf(plat, sizeof(plat) - 1, "PLATFORM='MAEMO'");
 #elif defined(SAILFISHOS)
 	snprintf(plat, sizeof(plat) - 1, "PLATFORM='SFOS'");
 #else
@@ -974,15 +1000,14 @@ static int instead_package(const char *path)
 	if (!stead_path)
 		return -1;
 	strcpy(stead_path, "package.path=\"");
-#if defined(_WIN32_WCE) || defined(WINRT)
 	if (path) {
-		strcat(stead_path, path); /* wince have not cwd :) */
+#if defined(WINRT)
+		strcat(stead_path, path); /* winrt have not cwd :) */
 		strcat(stead_path, "/?.lua;");
-	}
 #else
-	if (path)
 		strcat(stead_path, "./?.lua;");
 #endif
+	}
 
 #ifdef INSTEAD_LEGACY
 	p = instead_local_stead_path(wd);
@@ -1016,6 +1041,18 @@ const char *instead_get_api(void)
 	return API;
 }
 
+const char *instead_lua_path(const char *path)
+{
+	if (!path)
+		return instead_base_path;
+	if (!*path) {
+		strncpy(instead_base_path, STEAD_PATH, sizeof(instead_base_path) - 1);
+		return instead_base_path;
+	}
+	strncpy(instead_base_path, path, sizeof(instead_base_path) - 1);
+	return instead_base_path;
+}
+
 static int instead_set_api(const char *api)
 {
 	int i, c = 0;
@@ -1023,7 +1060,7 @@ static int instead_set_api(const char *api)
 	char *oa;
 	if (!api || !*api) {
 		FREE(API);
-		snprintf(instead_api_path, sizeof(instead_api_path), "%s", STEAD_PATH);
+		snprintf(instead_api_path, sizeof(instead_api_path), "%s", instead_lua_path(NULL));
 	} else {
 		s = strlen(api);
 		for (i = 0; i < s; i ++) {
@@ -1040,7 +1077,7 @@ static int instead_set_api(const char *api)
 		oa = API;
 		API = strdup(api);
 		FREE(oa);
-		snprintf(instead_api_path, sizeof(instead_api_path), "%s/%s", STEAD_PATH, API);
+		snprintf(instead_api_path, sizeof(instead_api_path), "%s/%s", instead_lua_path(NULL), API);
 	}
 	return 0;
 }
@@ -1101,6 +1138,7 @@ int instead_init_lua(const char *path, int detect)
 	setlocale(LC_COLLATE, "C");
 #endif
 /*	strcpy(curcp, "UTF-8"); */
+	instead_set_encoding("UTF-8");
 	getdir(instead_cwd_path, sizeof(instead_cwd_path));
 	unix_path(instead_cwd_path);
 	instead_cwd_path[sizeof(instead_cwd_path) - 1] = 0;
@@ -1185,7 +1223,7 @@ int instead_init(const char *path)
 		goto err;
 	}
 #ifdef _USE_SDL
-#ifndef __EMSCRIPTEN__
+#if !defined(__EMSCRIPTEN__) && !defined(WINRT)
 	sem = SDL_CreateMutex();
 	if (!sem)
 		goto err;
@@ -1236,7 +1274,7 @@ void instead_done(void)
 	if (wasL)
 		extensions_hook(done);
 #ifdef _USE_SDL
-#ifndef __EMSCRIPTEN__
+#if !defined(__EMSCRIPTEN__) && !defined(WINRT)
 	if (sem)
 		SDL_DestroyMutex(sem);
 	sem = NULL;
@@ -1244,6 +1282,7 @@ void instead_done(void)
 #endif
 #ifdef _HAVE_ICONV
 	FREE(fromcp);
+	FREE(curcp);
 #endif
 	if (L)
 		lua_close(L);
