@@ -1,5 +1,5 @@
 /*
- * Copyright 2009-2018 Peter Kosyh <p.kosyh at gmail.com>
+ * Copyright 2009-2021 Peter Kosyh <p.kosyh at gmail.com>
  *
  * Permission is hereby granted, free of charge, to any person
  * obtaining a copy of this software and associated documentation files
@@ -174,16 +174,30 @@ static int luaB_free_sprites(lua_State *L) {
 	return 0;
 }
 
+#define PIXELS_MAGIC 0x1980
+struct lua_pixels {
+	int type;
+	int w;
+	int h;
+	float scale;
+	size_t size;
+	img_t img;
+	int dirty;
+	int direct;
+};
+
 static int luaB_load_sprite(lua_State *L) {
+	int convert = 0;
 	img_t img = NULL;
 	_spr_t *sp;
 	const char *key;
 	char sname[sizeof(unsigned long) * 2 + 16];
 	struct lua_pixels *pixels = lua_touserdata(L, 1);
-	const char *desc = luaL_optstring(L, 2, NULL);
+	const char *desc = NULL;
 	const char *fname = NULL;
 	char pixels_name[32];
-
+	if (lua_isstring(L, 2))
+		desc = luaL_optstring(L, 2, NULL);
 	if (!pixels)
 		fname = luaL_optstring(L, 1, NULL);
 	else {
@@ -196,13 +210,23 @@ static int luaB_load_sprite(lua_State *L) {
 		return 0;
 
 	if (pixels) {
-		img = pixels_img(pixels);
-		if (img)
-			img = gfx_dup(img);
+		if (lua_isboolean(L, 2)) {
+			convert = lua_toboolean(L, 2);
+			desc = luaL_optstring(L, 3, NULL);
+		}
+		if (convert) { /* slow path */
+			img = gfx_new_from(pixels->w, pixels->h, (unsigned char*)(pixels + 1));
+			if (img)
+				theme_gfx_scale(&img, pixels->scale);
+		} else {
+			img = pixels_img(pixels);
+			if (img)
+				img = gfx_dup(img);
+		}
 	} else {
 		img = gfx_load_image((char*)fname);
 		if (img)
-			theme_img_scale(&img);
+			theme_gfx_scale(&img, 1.0f);
 	}
 	if (img)
 		img = gfx_display_alpha(img); /*speed up */
@@ -864,7 +888,9 @@ static int luaB_theme_var(lua_State *L) {
 	if (!opt_owntheme)
 		return 0;
 	if (!strcmp(var, "scr.w") ||
-		!strcmp(var, "scr.h")) /* filter resolution */
+		!strcmp(var, "scr.h") ||
+		!strcmp(var, "scr.scale_aware") ||
+		!strcmp(var, "scr.dpi")) /* filter resolution */
 		return 0;
 	if (!theme_setvar((char*)var, (char*)val)) {
 		if (strcmp(var, "win.scroll.mode")) /* let change scroll mode w/o theme reload */
@@ -1037,18 +1063,6 @@ static int luaB_get_themespath(lua_State *L) {
 	lua_pushstring(L, themes_path);
 	return 1;
 }
-#define PIXELS_MAGIC 0x1980
-struct lua_pixels {
-	int type;
-	int w;
-	int h;
-	float scale;
-	size_t size;
-	img_t img;
-	int dirty;
-	int direct;
-};
-
 static int pixels_size(lua_State *L) {
 	struct lua_pixels *hdr = (struct lua_pixels*)lua_touserdata(L, 1);
 	if (!hdr || hdr->type != PIXELS_MAGIC)
@@ -1560,7 +1574,7 @@ static void triangle(struct lua_pixels *src, int x0, int y0, int x1, int y1, int
 
 	for (y = miny; y <= maxy; y ++) {
 		int w0 = w0_row;
-	        int w1 = w1_row;
+		int w1 = w1_row;
 		int w2 = w2_row;
 		unsigned char *p = ptr;
 		for (x = minx; x <= maxx; x++) {
@@ -1777,7 +1791,7 @@ static int pixels_fill(lua_State *L) {
 		b = luaL_optnumber(L, 8, 0);
 		a = luaL_optnumber(L, 9, 255);
 	}
-	_fill(src, x, y, w, h, r, g, b, a, PXL_BLEND_BLEND);
+	_fill(src, x, y, w, h, r, g, b, a, (a == 255)?PXL_BLEND_COPY:PXL_BLEND_BLEND);
 	return 0;
 }
 
@@ -1820,6 +1834,12 @@ static int pixels_triangle(lua_State *L) {
 	y1 = luaL_optnumber(L, 5, 0);
 	x2 = luaL_optnumber(L, 6, 0);
 	y2 = luaL_optnumber(L, 7, 0);
+	#define XOR_SWAP(x,y) x=x^y; y=x^y; x=x^y;
+	if (orient2d(x0, y0, x1, y1, x2, y2) < 0) {
+		XOR_SWAP(x1, x2)
+		XOR_SWAP(y1, y2)
+	}
+	#undef XOR_SWAP
 	r = luaL_optnumber(L, 8, 0);
 	g = luaL_optnumber(L, 9, 0);
 	b = luaL_optnumber(L, 10, 0);
@@ -2509,6 +2529,11 @@ static int luaB_screen_size(lua_State *L) {
 	return 2;
 }
 
+static int luaB_screen_dpi(lua_State *L) {
+	lua_pushnumber(L, gfx_get_dpi());
+	return 1;
+}
+
 static const luaL_Reg sprites_funcs[] = {
 	{"instead_font_load", luaB_load_font},
 	{"instead_font_free", luaB_free_font},
@@ -2545,6 +2570,7 @@ static const luaL_Reg sprites_funcs[] = {
 	{"instead_noise3", luaB_noise3},
 	{"instead_noise4", luaB_noise4},
 	{"instead_screen_size", luaB_screen_size},
+	{"instead_screen_dpi", luaB_screen_dpi},
 	{"instead_render_callback", luaB_after_callback},
 	{NULL, NULL}
 };
