@@ -5,7 +5,7 @@ local std = stead
 local instead = std.obj {
 	nam = '@instead';
 
-	version_table = {3, 3, 2};
+	version_table = {3, 5, 0};
 
 	ini = function(s) -- after reset always do fade
 		s.need_fading(true)
@@ -77,7 +77,7 @@ local function get_bool(o, nam)
 		return o[nam]
 	end
 	if type(o[nam]) == 'function' then
-		return o:nam()
+		return o[nam](o)
 	end
 	return nil
 end
@@ -436,6 +436,10 @@ function iface:input(event, ...)
 		if type(input.click) == 'function' then
 			return input:click(...); -- pressed, x, y, mb
 		end
+	elseif event == 'wheel' then
+		if type(input.wheel) == 'function' then
+			return input:wheel(...);
+		end
 	elseif event == 'finger' then
 		if type(input.finger) == 'function' then
 			return input:finger(...); -- pressed, x, y, finger
